package com.qmth.themis.admin.api;

import java.util.Collections;
import java.util.List;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;

import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TEExamActivity;
import com.qmth.themis.business.service.TEExamActivityService;
import com.qmth.themis.business.service.TEExamPaperService;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;

/**
 * @Description: 考试场次 前端控制器
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Api(tags = "考试场次Controller")
@RestController
@RequestMapping("/${prefix.url.admin}/activity")
@Validated
public class TEExamActivityController {

    @Resource
    private TEExamActivityService teExamActivityService;
    
    @Resource
    private  TEExamPaperService examPaperService;

    @ApiOperation(value = "考试场次修改/新增接口")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @Transactional
    @ApiResponses({@ApiResponse(code = 200, message = "{\"success\":true}", response = Result.class)})
    public Result save(
            @ApiParam(value = "考试场次信息", required = true) @RequestBody List<TEExamActivity> teExamActivityList) {
    	teExamActivityService.saveExamActivity(teExamActivityList);
    	examPaperService.disposeObjectiveAnswer(teExamActivityList.get(0).getExamId());
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "考试场次查询接口")
    @RequestMapping(value = "/query", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "考试场次信息", response = TEExamActivity.class)})
    public Result query(@ApiParam(value = "主键", required = false) @RequestParam(required = false) Long id,
                        @ApiParam(value = "考试批次id", required = false) @RequestParam(required = false) Long examId,
                        @ApiParam(value = "考试场次编码", required = false) @RequestParam(required = false) String code,
                        @ApiParam(value = "开始日期", required = false) @RequestParam(required = false) Long startDate,
                        @ApiParam(value = "结束日期", required = false) @RequestParam(required = false) Long finishDate,
                        @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) int pageNumber,
                        @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) int pageSize) {
        return ResultUtil.ok(teExamActivityService
                .examActivityQuery(new Page<>(pageNumber, pageSize), id, examId, code, startDate, finishDate));
    }
}
