package com.qmth.themis.business.service.impl;

import com.qmth.themis.business.bean.exam.QrMobileMonitorParamBean;
import com.qmth.themis.business.bean.exam.QrResponseBean;
import com.qmth.themis.business.bean.exam.QrUploadParamBean;
import com.qmth.themis.business.cache.MobileAuthCacheUtil;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.enums.MobileModeEnum;
import com.qmth.themis.business.enums.RoleEnum;
import com.qmth.themis.business.service.TEQrcodeService;
import com.qmth.themis.business.util.QrCodeUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.business.util.WxappUtil;
import com.qmth.themis.common.exception.BusinessException;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.security.NoSuchAlgorithmException;
import java.util.Date;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;

@Service
public class TEQrcodeServiceImpl implements TEQrcodeService {

    @Resource
    RedisUtil redisUtil;

    @Resource
    WxappUtil wxappUtil;

    @Resource
    QrCodeUtil qrCodeUtil;

    @Override
    public QrResponseBean mobileMonitor(QrMobileMonitorParamBean param) throws NoSuchAlgorithmException {
        String domain = wxappUtil.getWxappDomain().getUrl();
        String code = SystemConstant.getUuid();
        MobileModeEnum mode = MobileModeEnum.MOBILE_MONITOR;
        Set<String> roleCodes = new HashSet<>();
        roleCodes.add(RoleEnum.STUDENT.name());
        MobileAuthCacheUtil.setUserType(mode, code, roleCodes.toString());
        MobileAuthCacheUtil.setMode(mode, code);
        MobileAuthCacheUtil.setCode(mode, code);
        MobileAuthCacheUtil.setRecordId(mode, code, param.getRecordId());
        MobileAuthCacheUtil.setMonitorVideoSource(mode, code, param.getSource());
        if (Objects.isNull(qrCodeUtil.getSysDomain().getQrExpireTime())) {
            throw new BusinessException("未设置二维码失效时间");
        }
        Long expireTime = -1L;
        if (qrCodeUtil.getSysDomain().getQrExpireTime() > 0) {
            expireTime = DateUtils.addSeconds(new Date(), qrCodeUtil.getSysDomain().getQrExpireTime()).getTime();
            redisUtil.expire(RedisKeyHelper.mobileAuthCacheKey(mode, code), qrCodeUtil.getSysDomain().getQrExpireTime());
        }
        QrResponseBean ret = new QrResponseBean();
        ret.setExpireTime(expireTime);
        ret.setContent(domain + "/" + mode.name().toLowerCase() + "/" + code);
        return ret;
    }

    @Override
    public QrResponseBean upload(QrUploadParamBean param, MobileModeEnum mode) {
        String domain = wxappUtil.getWxappDomain().getUrl();
        String code = SystemConstant.getUuid();
        MobileAuthCacheUtil.setMode(mode, code);
        MobileAuthCacheUtil.setCode(mode, code);
        MobileAuthCacheUtil.setRecordId(mode, code, param.getRecordId());
        MobileAuthCacheUtil.setMainNumber(mode, code, param.getMainNumber());
        MobileAuthCacheUtil.setSubNumber(mode, code, param.getSubNumber());
        MobileAuthCacheUtil.setQuestionNumber(mode, code, param.getQuestionNumber());
        if (param.getSubIndex() != null) {
            MobileAuthCacheUtil.setSubIndex(mode, code, param.getSubIndex());
        }
        if (Objects.isNull(qrCodeUtil.getSysDomain().getQrExpireTime())) {
            throw new BusinessException("未设置二维码失效时间");
        }
        Long expireTime = -1L;
        if (qrCodeUtil.getSysDomain().getQrExpireTime() > 0) {
            expireTime = DateUtils.addSeconds(new Date(), qrCodeUtil.getSysDomain().getQrExpireTime()).getTime();
            redisUtil.expire(RedisKeyHelper.mobileAuthCacheKey(mode, code), qrCodeUtil.getSysDomain().getQrExpireTime());
        }
        QrResponseBean ret = new QrResponseBean();
        ret.setExpireTime(expireTime);
        ret.setContent(domain + "/" + mode.name().toLowerCase() + "/" + code);
        return ret;
    }

}
